@@ -19,11 +19,15 @@ Functional tests for `shade` block storage methods.
 
 from testtools import content
 
+from shade import _utils
 from shade.tests.functional import base
 
 
 class TestVolume(base.BaseFunctionalTestCase):
 
+    # Creating and deleting volumes is slow
+    TIMEOUT_SCALING_FACTOR = 1.5
+
     def setUp(self):
         super(TestVolume, self).setUp()
         if not self.user_cloud.has_service('volume'):
@@ -77,7 +81,7 @@ class TestVolume(base.BaseFunctionalTestCase):
         self.user_cloud.delete_image(image_name, wait=True)
         self.user_cloud.delete_volume(volume_name, wait=True)
 
-    def cleanup(self, volume_name, snapshot_name=None, image_name=None):
+    def cleanup(self, volume, snapshot_name=None, image_name=None):
         # Need to delete snapshots before volumes
         if snapshot_name:
             snapshot = self.user_cloud.get_volume_snapshot(snapshot_name)
@@ -88,6 +92,44 @@ class TestVolume(base.BaseFunctionalTestCase):
             image = self.user_cloud.get_image(image_name)
             if image:
                 self.user_cloud.delete_image(image_name, wait=True)
-        volume = self.user_cloud.get_volume(volume_name)
-        if volume:
-            self.user_cloud.delete_volume(volume_name, wait=True)
+        if not isinstance(volume, list):
+            self.user_cloud.delete_volume(volume, wait=True)
+        else:
+            # We have more than one volume to clean up - submit all of the
+            # deletes without wait, then poll until none of them are found
+            # in the volume list anymore
+            for v in volume:
+                self.user_cloud.delete_volume(v, wait=False)
+            for count in _utils._iterate_timeout(
+                    180, "Timeout waiting for volume cleanup"):
+                found = False
+                for existing in self.user_cloud.list_volumes():
+                    for v in volume:
+                        if v['id'] == existing['id']:
+                            found = True
+                            break
+                    if found:
+                        break
+                if not found:
+                    break
+
+    def test_list_volumes_pagination(self):
+        '''Test pagination for list volumes functionality'''
+
+        volumes = []
+        # the number of created volumes needs to be higher than
+        # CONF.osapi_max_limit but not higher than volume quotas for
+        # the test user in the tenant(default quotas is set to 10)
+        num_volumes = 8
+        for i in range(num_volumes):
+            name = self.getUniqueString()
+            v = self.user_cloud.create_volume(display_name=name, size=1)
+            volumes.append(v)
+        self.addCleanup(self.cleanup, volumes)
+        result = []
+        for i in self.user_cloud.list_volumes():
+            if i['name'] and i['name'].startswith(self.id()):
+                result.append(i['id'])
+        self.assertEqual(
+            sorted([i['id'] for i in volumes]),
+            sorted(result))
