@@ -301,7 +301,7 @@ class TestImage(BaseTestImage):
         self.assert_calls()
 
     def _image_dict(self, fake_image):
-        return self.cloud._normalize_image(meta.obj_to_dict(fake_image))
+        return self.cloud._normalize_image(meta.obj_to_munch(fake_image))
 
     def _munch_images(self, fake_image):
         return self.cloud._normalize_images([fake_image])
